# encoding: utf-8

from datetime import datetime


__author__ = "Tsuyoshi Hombashi"
__copyright__ = "Copyright 2016-{}, {}".format(datetime.now().year, __author__)
__license__ = "MIT License"
__version__ = "0.21.9"
__maintainer__ = __author__
__email__ = "tsuyoshi.hombashi@gmail.com"
